@@ -1,17 +1,20 @@
 # %%
 
 from . import extra_funcs as ef
+#import extra_funcs as ef
 from .units import unit
+#from units import unit
 from copy import deepcopy
 import inspect
 import matplotlib.pyplot as plt
+from matplotlib.offsetbox import AnchoredText
 import numpy as np
 import os
 import pandas as pd
 from scipy.optimize import curve_fit
 import sympy as sp
 
-exval, exUnis = {}, {}
+exVals, exUnis = {}, {}
 
 showUncertain = True
 
@@ -64,9 +67,11 @@ class table:
     (data) if you do not want to import from a data file, you can pass a
         np.array to this argument. The datname you specified will be ignored.
     """
+
     def __init__(self, datname, AutoInsert=True, units={}, sheet=None,
-                 data=None, delimiter=',', skiprows=None, noNames=False):
-        self.units = units
+                 data=None, delimiter=',', skiprows=None, noNames=False,
+                 comment='#'):
+        self.units = deepcopy(units)
         if data is not None:
             if isinstance(data, pd.DataFrame):
                 self.data = data
@@ -96,15 +101,14 @@ class table:
                 self.data = pd.read_excel(name + '.' + ext, sheet_name=sheet)
             else:
                 self.data = pd.read_csv(name + '.' + ext, delimiter=delimiter,
-                                        skiprows=skiprows)
+                                        skiprows=skiprows, comment=comment)
                 if noNames:
                     names = list(map(str, range(len(self.data.columns))))
                     self.data = pd.read_csv(name + '.' + ext,
                                             delimiter=delimiter,
                                             skiprows=skiprows, header=0,
-                                            names=names)
+                                            names=names, comment=comment)
 
-        self.giveUnits(units)
         self.formulas = {}
 
         if AutoInsert:
@@ -121,11 +125,14 @@ class table:
                         self.data.insert(i + 1, ef.delt(self.data.columns[i]),
                                          [0 for i in range(len(self.data))])
                         lol = True
-                        pass
+
             if not ef.isUncertain(*self.data.columns[-2:]):
                 self.data.insert(len(self.data.columns), ef.delt(
                     self.data.columns[-1]), [0 for i in range(len(self.data))])
         self.data.columns = list(map(str, self.data.columns))
+        for i in self.data.columns:
+            self.units[i] = unit('1')
+        self.giveUnits(units)
 
     def __repr__(self):
         dat = [c for c in list(self.data) if 'Delta' not in c]
@@ -195,7 +202,7 @@ class table:
             pre_dict_i = dict(zip(
                 [f"dummy{x}" for x in range(len(expr_incert.free_symbols))],
                 map(str, expr_incert.free_symbols)
-                ))
+            ))
             expr_incert = expr_incert.subs(
                 {v: k for k, v in pre_dict_i.items()})
             lamb_incert = sp.lambdify(tuple(expr_incert.free_symbols),
@@ -203,20 +210,20 @@ class table:
         # Le cacul est fait ligne par ligne, probablement très optimisable
         for i in range(len(self.data)):
             vals = V_dict[i]
-            vals.update(exval)
+            vals.update(exVals)
             nvals = {name: vals[pre_dict[name]] for name in pre_dict}
             delt_vals = {name: vals[pre_dict_i[name]] for name in pre_dict_i}
             out[i] = [
                 lamb(**nvals), 0 if NoIncert else lamb_incert(**delt_vals)]
-        # Nomme les colonnes et les ajoute au tableau
+        # nameme les colonnes et les ajoute au tableau
         if units:
             self.units[name] = unit(units)
         else:
             self.units[name] = lamb(*(
                 self.units[str(i)]
                 if str(i) in self.units
-                else exUnis[str(i)]for i in self.formulas[name].free_symbols)
-                )
+                else deepcopy(exUnis[str(i)]) for i in self.formulas[name].free_symbols)
+            )
         self.data.insert(pos, name, out[:, 0])
         self.data.insert(pos + 1, ef.delt(name), out[:, 1])
         self.fixUnits()
@@ -248,10 +255,10 @@ class table:
             try:
                 if isinstance(units[col], str):
                     units[col] = unit(units[col])
-                self.units[col] = units[col]
+                self.units[col] = deepcopy(units[col])
             except:
                 try:
-                    self.units[col] = self.units[col]
+                    self.units[col] = deepcopy(self.units[col])
                 except:
                     self.units[col] = unit("1")
 
@@ -281,9 +288,15 @@ class table:
         """
         # vars = list(sp.symbols(noms))
         if isinstance(names, str):
-            self.data.columns = [n for var in names.split(" ")
+            names = names.split(" ")
+            for i in range(len(names)):
+                self.units[names[i]] = deepcopy(self.units[self.data.columns[::2][i]])
+            self.data.columns = [n for var in names
                                  for n in (var, ef.delt(var))]
+
         if isinstance(names, dict):
+            for key in list(names.keys()):
+                self.units[names[key]] = deepcopy(self.units[key])
             keys = [n for var in names.keys()
                     for n in (var, ef.delt(var))]
             vals = [n for var in names.values()
@@ -291,6 +304,8 @@ class table:
             self.data = self.data.rename(columns=dict(zip(keys, vals)))
 
         if isinstance(names, list):
+            for i in range(len(names)):
+                self.units[names[i]] = deepcopy(self.units[self.data.columns[::2][i]])
             self.data.columns =\
                 [n for var in names for n in (var, ef.delt(var))]
 
@@ -312,12 +327,12 @@ class table:
                     outCol.append("$" + s + "$")
                 else:
                     miam = "{{:.{}g}}"\
-                          .format(- np.ceil(-sp.log(abs(val), 10))
-                                  + np.ceil(-sp.log(d, 10)) + 1).format(val)
+                        .format(- np.ceil(-sp.log(abs(val), 10))
+                                + np.ceil(-sp.log(d, 10)) + 1).format(val)
                     outCol.append(
                         "$" + miam + " \\pm " + "{:.1g}$"
                         .format(ef.roundUp(self.data[ef.delt(col)][x]))
-                        )
+                    )
             out["$" + col + "$"] = outCol
         return(out)
 
@@ -415,7 +430,7 @@ class table:
         ax.set_ylabel(ef.ax_name(dt, yn))
 
     def fit(self, func, xn, yn, fig_ax=None, show=True, p0=None, maxfev=1000,
-            fit_label='fit', fit_color=None, **kwargs):
+            fit_label='fit', fit_color=None, showEq=False, **kwargs):
         """\n
 
         return optimal parameters of a function fitted on data
@@ -439,23 +454,37 @@ class table:
         if fig_ax is None:
             fig_ax = plt.figure(), plt.gca()
         fig, ax = fig_ax
-        popt, cov = curve_fit(func, *self[[xn, yn]].T, p0=p0, maxfev=maxfev)
+        dt = self.copy()
+        if xn not in self.data.columns:
+            dt.newCol(xn, xn)
+        if yn not in self.data.columns:
+            dt.newCol(yn, yn)
+        popt, cov = curve_fit(func, *dt[[xn, yn]].T, p0=p0, maxfev=maxfev)
         res = np.array([np.array([popt, np.sqrt(cov.diagonal())]).T.flatten()])
-        dt = table('', data=res, AutoInsert=False)
-        noms = list(inspect.signature(func).parameters.keys())
-        noms = list(map(lambda x: sp.latex(sp.sympify(ef.preSymp(x))), noms))
-        dt.renameCols(noms[1:])
+        out = table('', data=res, AutoInsert=False)
+        names = list(inspect.signature(func).parameters.keys())
+        names = list(map(lambda x: sp.latex(sp.sympify(ef.preSymp(x))), names))
+        out.renameCols(names[1:])
         lines = inspect.getsourcelines(func)
 
-        if len(lines) == 2 and fit_label == 'fit':
+        if len(lines) == 2:
             expr = sp.sympify(ef.preSymp(lines[0][-1].replace('return ', '')))
-            fit_label = "$"+sp.latex(expr)+"$"
+            if fit_label == 'fit':
+                fit_label = "$"+sp.latex(expr)+"$"
+            if showEq:
+                values = {name: ef.precisionStr(out[name][0], out[ef.delt(name)][0])
+                                for name in names[1:]}
+                eq = sp.latex(expr)
+                for name, val in values.items():
+                    eq = eq.replace(str(name), val)
+                eq = eq.replace(names[0], ef.latexify(xn))
+                ax.add_artist(AnchoredText('$'+ef.latexify(yn)+r' \approx '+eq+'$', loc=2))
         if show:
-            x = np.linspace(*ef.extrem(self[xn]), 1000)
+            x = np.linspace(*ef.extrem(dt[xn]), 1000)
             ax.plot(x, func(x, *popt), label=fit_label, color=fit_color)
-            self.plot(xn, yn, fig_ax=fig_ax, **kwargs)
+            dt.plot(xn, yn, fig_ax=fig_ax, **kwargs)
             ax.legend()
-        return dt
+        return out
 
     def append(self, name, data, incert=None, pos=None, units="1"):
         if pos is None:
@@ -469,7 +498,7 @@ class table:
 
     def copy(self):
         return deepcopy(self)
-    
+
     def getStr(self, cols):
         """Combines data and uncertainty collumns. Used with makeGoodTable"""
         cols = list(cols)
@@ -491,7 +520,7 @@ class table:
                     outCol.append(
                         "$" + ef.precisionStr(val, d) + " \\pm " + "{:.1g}$"
                         .format(ef.roundUp(self.data[ef.delt(col)][x]))
-                        )
+                    )
             out[col] = outCol
         if len(list(out.values())[0]) == 1:
             out = {key: out[key][0] for key in list(out.keys())}
@@ -502,8 +531,70 @@ class table:
 
 def defVal(vals):
     for i in vals:
-        a = vals[i].split(" ")
-        exval[i] = float(a[0])
-        exval[ef.delt(i)] = float(a[1])
+        if isinstance(i, str):
+            a = vals[i].split(" ")
+        else:
+            a = vals[i]
+        exVals[i] = float(a[0])
+        exVals[ef.delt(i)] = float(a[1])
         exUnis[i] = unit(a[2])
 
+
+def genVal(name, formula, units=None, NoUncert=False):
+    expression = sp.sympify(ef.preSymp(formula))
+    expression_orig = deepcopy(expression)
+    pre_dict = dict(zip([f"dummy{x}" for x in range(
+        len(expression.free_symbols))], map(str, expression.free_symbols)))
+    expression = expression.subs({v: k for k, v in pre_dict.items()})
+    lamb = sp.lambdify(tuple(expression.free_symbols), expression)
+    if not NoUncert:
+        expr_incert = ef.formule_incertitude(ef.preSymp(formula))
+        pre_dict_i = dict(zip(
+            [f"dummy{x}" for x in range(len(expr_incert.free_symbols))],
+            map(str, expr_incert.free_symbols)
+        ))
+        expr_incert = expr_incert.subs(
+            {v: k for k, v in pre_dict_i.items()})
+        lamb_incert = sp.lambdify(tuple(expr_incert.free_symbols),
+                                      expr_incert)
+    # Le cacul est fait ligne par ligne, probablement très optimisable
+    vals = exVals
+    nvals = {name: vals[pre_dict[name]] for name in pre_dict}
+    delt_vals = {name: vals[pre_dict_i[name]] for name in pre_dict_i}
+    exVals[name], exVals[ef.delt(name)] =\
+        [lamb(**nvals), 0 if NoUncert else lamb_incert(**delt_vals)]
+
+    if units:
+            exUnis[name] = unit(units)
+    else:
+        exUnis[name] = lamb(*(
+            exUnis[str(i)] for i in expression_orig.free_symbols))
+        # const = exUnis[name].exctractConstant()
+        # exVals[name] *= float(const)
+        # exVals[ef.delt(name)] *= float(const)
+        # exUnis[name] = unit(str(exUnis[name].symb / const))
+
+
+
+def getVal(name):
+    return exVals[name], exVals[ef.delt(name)], exUnis[name]
+
+
+def getStrVal(name, latex=True, sci=False):
+    val = getVal(name)
+    if latex:
+        s =  f"${ef.valueStr(*val[0:2], sci=sci)}$"
+        if not str(val[2]) == '1':
+            s += r" $\rm {:}$".format(exUnis[name])
+        return s
+    else:
+        s = f"{ef.valueStr(*val[0,1], sci=sci)}"
+        if not str(val[2]) == '1':
+            s += r" ( {:} )".format(tab.units[qty])
+        return s
+
+def changeUnitsVal(name, unit):
+    fact = exUnis[name].to(unit)
+    exVals[name] *= float(fact)
+    exVals[ef.delt(name)] *= float(fact)
+# %%
